/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces;

import org.ajax4jsf.test.base.RichSeleniumTest;
import org.ajax4jsf.test.base.SeleniumTestBase;
import org.ajax4jsf.test.base.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class ContextMenuTest extends SeleniumTestBase implements RichSeleniumTest {
	
	public ContextMenuTest() {
		super("http", "localhost", "8080");
	}

	/**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
    	super.startSelenium(browser);
    }
    
    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
    	super.stopSelenium();
    }
    
    @Test
    public void testContextMenuComponent() throws Exception {
    	_testContextMenuComponent(Templates.SIMPLE);
    	_testContextMenuComponent(Templates.DATATABLE);
    	_testContextMenuComponent(Templates.MODALPANEL);
    }

	private void _testContextMenuComponent(Templates template) {
		renderPage(getTestUrl(), template);
		
		String parentId = getParentId() + "_form:";
		String inputId = parentId + "input";
		String input2Id = parentId + "input2";
		String outputId = parentId + "_value2";
		String menuId = parentId + "menu_input_menu";
		String menu2Id = parentId + "menu_input2_menu";
		
		String menu1_item_ajax = parentId + "menu1_item1";
		String menu1_item_server = parentId + "menu1_item2";
		String menu2_item_ajax = parentId + "menu2_item1";
		String menu2_item_server = parentId + "menu2_item2";
		
		writeStatus("Click on first input");
		clickById(inputId);
		Assert.assertTrue(isVisibleById(menuId));
		
		writeStatus("Click on second input");
		clickById(input2Id);
		Assert.assertFalse(isVisibleById(menuId));
		Assert.assertTrue(isVisibleById(menu2Id));
		
		writeStatus("Click on first input again");
		clickById(inputId);
		Assert.assertTrue(isVisibleById(menuId));
		Assert.assertFalse(isVisibleById(menu2Id));
		
		writeStatus("Click on ajax item in first menu");
		clickById(menu1_item_ajax);
        waitForAjaxCompletion();
        AssertValueEquals(inputId, "menu1_item1");
        AssertTextEquals(outputId, "1");
		
        writeStatus("Click on server item in first menu");
        clickCommandAndWait(menu1_item_server);
        AssertValueEquals(inputId, "menu1_item2");
        AssertTextEquals(outputId, "2");
        
        writeStatus("Click on second input again");
		clickById(input2Id);
		Assert.assertTrue(isVisibleById(menu2Id));
		
		writeStatus("Click on ajax item in first menu");
		clickById(menu2_item_ajax);
        waitForAjaxCompletion();
        AssertValueEquals(inputId, "menu2_item1");
        AssertTextEquals(outputId, "3");
		
        writeStatus("Click on server item in second menu");
        clickCommandAndWait(menu2_item_server);
        AssertValueEquals(inputId, "menu2_item2");
        AssertTextEquals(outputId, "4");
	}
	
	public String getTestUrl() {
		return "/faces/pages/contextMenu/contextMenu.xhtml";
	}

}

